package chapter1.secondSample2;

import chapter1.secondSample2.GrailNotFoundException;
import chapter1.secondSample2.HolyGrail;
import chapter1.secondSample2.KnightOfTheRoundTable;
import junit.framework.TestCase;
import static org.easymock.EasyMock.*;
/**
 * Quest 인터페이스를 사용했기 때문에
 * 1. 종속성을 가지는 인터페이스(Quest)의 Mock 객체를 만들어서 단위 테스트를 할 수 있다.
 * 2. 여러 Mock 객체를 만들어서 다양한 상황을 테스트 할 수 있다.
**/
public class KnightOfTheRoundTableTest extends TestCase {


	//Best Case :: embarkOnQuest()에서 HolyGrail 반환 받는 경우.
	public void testEmbarkOnQuest() throws GrailNotFoundException {
		KnightOfTheRoundTable knight = new KnightOfTheRoundTable("Bedivere");

		Quest mockQuest = createMock(Quest.class);
		expect(mockQuest.embark()).andReturn(new HolyGrail());
		replay(mockQuest);

		knight.setQuest(mockQuest);
		HolyGrail grail = (HolyGrail) knight.embarkOnQuest();
		assertNotNull(grail);
		assertTrue(grail.isHoly());
	}

	//Bad Case :: embarkOnQuest()에서 null 반환 받는 경우.
	public void testEmbarkObQuest2() throws Exception {
		KnightOfTheRoundTable knight = new KnightOfTheRoundTable("Bedivere");

		Quest mockQuest = createMock(Quest.class);
		expect(mockQuest.embark()).andReturn(null);
		replay(mockQuest);

		knight.setQuest(mockQuest);
		Object grail = knight.embarkOnQuest();
		assertNull(grail);
	}

	//Bad Case :: embarkOnQuest()에서 GrailNotFoundException 예외 발생
	public void testEmbarkOnQuest3() throws Exception {
		KnightOfTheRoundTable knight = new KnightOfTheRoundTable("Bedivere");

		Quest mockQuest = createMock(Quest.class);
		expect(mockQuest.embark()).andThrow(new GrailNotFoundException());
		replay(mockQuest);

		knight.setQuest(mockQuest);
		try{
			knight.embarkOnQuest();
			fail();
		} catch (GrailNotFoundException e) {

		}
	}
}
